/*
    Based on Neil Kolban example for IDF: https://github.com/nkolban/esp32-snippets/blob/master/cpp_utils/tests/BLE%20Tests/SampleServer.cpp
    Ported to Arduino ESP32 by Evandro Copercini
    updates by chegewara
*/

#include <BLEDevice.h>
#include <BLEUtils.h>
#include <BLEServer.h>
#include <BLECharacteristic.h>
#include <BLE2902.h>
// See the following for generating UUIDs:
// https://www.uuidgenerator.net/

//FxChr20, Address: ed:2b:c9:74:87:6a, appearance: 0, serviceUUID: 
#define PIN_INPUT 0

const char * CHARACTERISTIC_UUID[] = {  "00001624-88EC-688C-644B-3FA706C0BB75",
                                        "00001625-88EC-688C-644B-3FA706C0BB75",
                                        "00001626-88EC-688C-644B-3FA706C0BB75",
                                        "00001627-88EC-688C-644B-3FA706C0BB75",
                                        "00001628-88EC-688C-644B-3FA706C0BB75",
                                        "00001629-88EC-688C-644B-3FA706C0BB75",
                                        "0000162A-88EC-688C-644B-3FA706C0BB75"};



typedef struct fx_values {
  uint8_t * p;
  size_t l;
} fx_values_t;

uint8_t fx_bytes[10] = {0,0,0, 0, 0x00, 
  0xC3,       //Battery mv / 20
  0x00, 0, 0x74};
//uint8_t fx_bytes[10] = {0,0,0, 0, 0x32, 0xD3, 0, 0, 0x96};

fx_values_t the_values[] = {
  {&fx_bytes[0], 3},
  {&fx_bytes[3], 1},
  {&fx_bytes[4], 1},
  {&fx_bytes[5], 1},
  {&fx_bytes[6], 1},
  {&fx_bytes[7], 1},
  {&fx_bytes[8], 1}
};

BLECharacteristic *pCharacteristic[7];

/* 
0x000000
0x00
0x32
0xD3
0x00
0x00
0x96

//highest minimum return
W: UUID: 00001626 64 
W: UUID: 00001628 5A 
R: UUID: 00001627 
R: UUID: 00001626 
R: UUID: 00001628 

46-141 M/S
60-181
90-220
134-250
180-335

bow/airsoft
W: UUID: 00001626 32 
W: UUID: 00001628 17 
CO2 Pistol
W: UUID: 00001626 32 
W: UUID: 00001628 1E
Air Pistol
W: UUID: 00001626 32 
W: UUID: 00001628 2D
Air gun UK
W: UUID: 00001626 32 
W: UUID: 00001628 43
Air gun High Power
W: UUID: 00001626 64
W: UUID: 00001628 5A




W: UUID: 00001626   64 
W: UUID: 00001628   43 
W: UUID: 0000162a   74 
W: UUID: 00001625   00 



R: UUID: 00001627   CC
R: UUID: 00001626   64
R: UUID: 00001628   43
R: UUID: 0000162a   74
R: UUID: 00001625   00

*/

bool deviceConnected;

class MyServerCallbacks: public BLEServerCallbacks {
    void onConnect(BLEServer* pServer) {
      Serial.print("Connected");
      deviceConnected = true;
    };

    void onDisconnect(BLEServer* pServer) {
      deviceConnected = false;
   BLEDevice::startAdvertising();
     Serial.print("Disconnected");
    }
};

class CharCallbacks: public BLECharacteristicCallbacks {
  void onWrite(BLECharacteristic *pCharacteristic) 
  {
    std::string value = pCharacteristic->getValue();
    Serial.printf("W: %s %02X\n",pCharacteristic->toString().c_str(),value.c_str());
    Serial.printf("%02X ",value[0]);

    Serial.println();
  }

  void onRead	(	BLECharacteristic * pCharacteristic)	
  {
    Serial.printf("R: %s %02X\n",pCharacteristic->toString().c_str(),pCharacteristic->getValue().c_str());
    for(int i=0;i<10;i++){
      Serial.printf("%02X ",fx_bytes[i]);
    }
    Serial.println();
  }
};

BLEDescriptor descriptor(BLEUUID((uint16_t)0x2902));

void setup() {
  Serial.begin(115200);
  Serial.println("Starting BLE work!");
  pinMode(PIN_INPUT, INPUT);

  BLEDevice::init("FxChr20");
  BLEServer *pServer = BLEDevice::createServer();
  pServer->setCallbacks(new MyServerCallbacks());
  BLEService *pService = pServer->createService("00001623-88EC-688C-644B-3FA706C0BB75");
  for(int i = 0; i < 7; i++) {
    pCharacteristic[i] = pService->createCharacteristic(
                                          CHARACTERISTIC_UUID[i],
                                          BLECharacteristic::PROPERTY_READ |
                                          (i == 0 ? BLECharacteristic::PROPERTY_NOTIFY : BLECharacteristic::PROPERTY_WRITE)
                                        );
    pCharacteristic[i]->setCallbacks(new CharCallbacks());

    pCharacteristic[i]->setValue(the_values[i].p, the_values[i].l);

    if(i == 0) {
      pCharacteristic[i]->addDescriptor(&descriptor);
    }

    pService->start();

    // BLEAdvertising *pAdvertising = pServer->getAdvertising();  // this still is working for backward compatibility

  }

  BLEAdvertising *pAdvertising = BLEDevice::getAdvertising();
  pAdvertising->addServiceUUID("0000180a-0000-1000-8000-00805f9b34fb");

  pAdvertising->setScanResponse(true);
  pAdvertising->setMinPreferred(0x06);  // functions that help with iPhone connections issue
  pAdvertising->setMinPreferred(0x12);
  BLEDevice::startAdvertising();
  Serial.println("Characteristic defined! Now you can read it in your phone!");
}

uint8_t nvalue[3] = {0,0,0};
uint32_t ctr = 0;
uint8_t sctr=0;

uint8_t v = 0;

void loop() {
  // put your main code here, to run repeatedly:
  delay(200);

  // byte 0   250 = 927M/S   3.708 Meters/count   
  // byte 1   
  // byte 3 - radar return  0-20 (0-100%) Must be > threshold in app for shot to count

  if(digitalRead(PIN_INPUT) == 0) Serial.println("ooh");


  if (deviceConnected) { 
    // 100,0,0 = 1216 fps
    if((ctr > 5) && (digitalRead(PIN_INPUT) == 0)) {
      nvalue[0] = 0x61;
      nvalue[1] = sctr;
      nvalue[2] = 15;
      sctr += 0x32;
      ctr = 0;
    } else {
      nvalue[0] = 0;
      nvalue[1] = 0;
      nvalue[2] = 0;
    }
    pCharacteristic[0]->setValue(nvalue, 3); 
    ctr++;
    
    pCharacteristic[0]->notify(); 
    //pCharacteristic->indicate(); 
  }
}